/*
** Matrix.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Sun Jan 27 09:44:38 2008 damien
** Last update Sun Jan 27 09:44:38 2008 damien
*/

#ifndef   	MATRIX_H_
# define   	MATRIX_H_

#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>
using namespace std;

#include "V_3D.h"
#include <vector>


class Matrix
{

 public:
  Matrix();
  Matrix(const Matrix& _M);
  ~Matrix();

  Matrix multiply(Matrix M2);
  double * multiply(double *x);
  int multiply_internally(double *x);
  int multiply_rotation_internally(double *normal_copy);

  //Matrix get_inverse();
  double det() const;

  int set_identity();
  int set_zero();

  int set_translation(double *t);
  int set_translation(double tx,double ty,double tz);
  int add_translation(double *t);
  int add_translation(double tx,double ty,double tz);

  int set_euler_angle(double *r);
  int set_euler_angle(double theta_1,double theta_2,double theta_3);

  int set_rotation_matrix(double *axis,double angle);
  int set_rotation_matrix(double x_axis,double y_axis,double z_axis,double angle);
  Matrix multiply_rotation(double *axis,double angle);
  Matrix multiply_rotation(double x_axis,double y_axis,double z_axis,double angle);

  /// build rotation from vector v to the vector w (vector are
  /// automatically normalized)
  /** Get rotation matrix R such that R*v=w */
  int build_rotation_matrix_to_vector(const V_3D v,const V_3D w);


  double value(int k_x,int k_y) const; 
  double& value(int k_x,int k_y); 
  double v(int k_x,int k_y) const; //shorthand for value
  double& v(int k_x,int k_y); 

  int set_value(double val,int k_x,int k_y);
  int set_value(double,double,double,double , double,double,double,double , double,double,double,double , double,double,double,double);

  int set_vector_rotation(double *v,double theta);//theta in radians
  int set_vector_rotation(double v_x,double v_y,double v_z,double theta);//theta in radians
  int multiply_vector_rotation(double *v,double theta);

  Matrix get_rotation_part();
  int set_rotation_part(Matrix R);
  int set_translation_part(Matrix T);

  friend ostream& operator << (ostream& flux, Matrix M);
  Matrix& operator=(const Matrix&);
  friend Matrix operator*(const Matrix&,const Matrix&);
  friend Matrix operator*(const Matrix&,const double&);
  friend Matrix operator*(const double&,const Matrix&);
  friend V_3D operator*(const Matrix&,const V_3D&);
  friend V_3D operator*(const V_3D& ,const Matrix&);
  friend std::vector <double> operator*(const std::vector <double>& ,const Matrix& );
  friend std::vector <double> operator*(const Matrix& ,const std::vector <double>& );
  friend Matrix operator+(const Matrix&,const Matrix&);
  friend Matrix operator-(const Matrix&,const Matrix&);
  friend int operator==(const Matrix&,const Matrix&);
  friend int operator!=(const Matrix&,const Matrix&);
  Matrix& operator+=(const Matrix&);
  Matrix& operator/=(const double&);
  double& operator [](int k_dim);
  double operator [](int k_dim) const;

  /// Return matrix(k_row,k_column)
  double operator ()(int k_1,int k_2) const;
  double& operator()(int k_1,int k_2);

  Matrix invert() const;

  /// transpose the matrix
  Matrix transpose() const;

  int get_position(double *p);
  V_3D get_position();

  int scale(double *s);

  double* get_pointer();

 private:

  double M[16];



};

#endif 	    /* !MATRIX_H_ */
