
// RICM 3 - VM
// Encapsulation class for Volume data

#include "Volume.h"




Volume::Volume()
{for(int k_dim=0;k_dim<3;k_dim++){N[k_dim]=-1;}}

Volume::~Volume()
{destroy();}

int Volume::destroy()
{
  data.resize(0);
  for(int k_dim=0;k_dim<3;k_dim++){N[k_dim]=-1;}
  return 0;
}

int Volume::load_v4d(const std::string& filename)
{
  destroy();

  FILE *fid=NULL;
  fid=fopen(filename.data(),"r");

  #define SIZE_BUFFER 2048
  char buffer[SIZE_BUFFER]={'\0'};
  
  if(fid==NULL){printf("cannot load file %s\n",filename.data());exit(-1);}
  for(int k_dim=0;k_dim<3;k_dim++)
    {
      fscanf(fid,"%s",buffer);
      N[k_dim]=atoi(buffer);
    }
  int k_0=0,k_1=0,k_2=0;

  float temp=0;
  data.resize(N[0]*N[1]*N[2]);
  for(k_0=0;k_0<N[0];k_0++)
    for(k_1=0;k_1<N[1];k_1++)
      for(k_2=0;k_2<N[2];k_2++)
	{
	  if(fscanf(fid,"%f",&temp)!=1){printf("Error reading file %s\n",filename.data());exit(-1);}
	  data[k_0+k_1*N[0]+k_2*N[0]*N[1]]=temp;
	}

  if(fclose(fid)!=0)
    {printf("Error in closing file %s to load it\n",filename.data());exit(-1);}

  printf("<File %s loaded, size=(%d x %d x %d)>\n",filename.data(),N[0],N[1],N[2]);
  return 0;
}
int Volume::load_v4d(const char* filename)
{return load_v4d(std::string(filename));}



int Volume::save_v4d(const std::string& filename) const
{

  FILE *fid=NULL;
  fid=fopen(filename.data(),"w");

  if(fid==NULL){printf("cannot load file %s to save it\n",filename.data());exit(-1);}
  for(int k_dim=0;k_dim<3;k_dim++)
    fprintf(fid,"%d ",N[k_dim]);
  fprintf(fid,"\n");

  int k_0=0,k_1=0,k_2=0;
  for(k_0=0;k_0<N[0];k_0++)
    for(k_1=0;k_1<N[1];k_1++)
      for(k_2=0;k_2<N[2];k_2++)
	fprintf(fid,"%f\n",data[k_0+k_1*N[0]+k_2*N[0]*N[1]]);
	  
  if(fclose(fid)!=0)
    {printf("Error in closing file %s to save it\n",filename.data());exit(-1);}

  printf("<File %s Saved, size=(%d x %d x %d)>\n",filename.data(),N[0],N[1],N[2]);
  return 0;
}
int Volume::save_v4d(const char* filename) const
{return save_v4d(std::string(filename));}

double Volume::get(int k_x,int k_y,int k_z) const
{
  if(k_x<0 || k_x>=N[0] || k_y<0 || k_y>=N[1] || k_z<0 || k_z>=N[2])
    {
      printf("Error in get(%d,%d,%d) in Volume, size=[%d,%d,%d]",k_x,k_y,k_z,N[0],N[1],N[2]);
      exit(-1);
    }
  return data[k_x+N[0]*k_y+N[0]*N[1]*k_z];
}
double& Volume::get(int k_x,int k_y,int k_z)
{
  if(k_x<0 || k_x>=N[0] || k_y<0 || k_y>=N[1] || k_z<0 || k_z>=N[2])
    {
      printf("Error in get(%d,%d,%d) in Volume, size=[%d,%d,%d]",k_x,k_y,k_z,N[0],N[1],N[2]);
      exit(-1);
    }
  return data[k_x+N[0]*k_y+N[0]*N[1]*k_z];
}

int Volume::get_size(int k_dim)
{
  if(k_dim<0 || k_dim>=3)
    {printf("Error in get_size(%d) in Volume\n",k_dim);exit(-1);}
  return N[k_dim];
}

const std::vector <double>& Volume::get() const
{return data;}
std::vector <double>& Volume::get()
{return data;}

int Volume::slice(const V_3D& n,const V_3D& p,int N1,int N2,double dx,double dy,std::vector <double>* vertex,std::vector <int>* connectivity,std::vector <double>* color) const
{
  if(vertex==NULL || connectivity==NULL || color==NULL)
    {printf("Null pointer in slice in Volume\n");exit(-1);}

  std::vector <double> current_slice(N1*N2);
  Matrix Rotation; Rotation.build_rotation_matrix_to_vector(V_3D(0,0,1),n);
  V_3D x;
  V_3D current_position;

  int k1=0,k2=0;
  for(k2=0;k2<N2;k2++)
    for(k1=0;k1<N1;k1++)
      {
	x.set(dx*double(k1-N1/2),dy*double(k2-N2/2),0.0);
	current_position = Rotation*x+p;
	
	for(int k_dim=0;k_dim<3;k_dim++)
	  vertex->push_back(current_position[k_dim]);
	
	color->push_back(get(current_position[0],current_position[1],current_position[2]));
      }

  //connectivity
  for(k2=0;k2<N2-1;k2++)
    for(k1=0;k1<N1-1;k1++)
      {
	//triangle 1
	connectivity->push_back((k1+0)+N1*(k2+0));
	connectivity->push_back((k1+1)+N1*(k2+0));
	connectivity->push_back((k1+1)+N1*(k2+1));

	//triangle 2
	connectivity->push_back((k1+0)+N1*(k2+0));
	connectivity->push_back((k1+1)+N1*(k2+1));
	connectivity->push_back((k1+0)+N1*(k2+1));
      }

  return 0;
}

std::vector <double> Volume::slice(const V_3D& n,const V_3D& p,int N1,int N2,double dx,double dy) const
{
  std::vector <double> vertex;
  std::vector <int> connectivity;
  std::vector <double> color;
  slice(n,p,N1,N2,dx,dy,&vertex,&connectivity,&color);
  return color;
}

double Volume::get(double x,double y,double z) const
{
  int k_x=0,k_y=0,k_z=0;
  k_x=int(x);
  k_y=int(y);
  k_z=int(z);

  if(k_x<0 || k_x>=N[0] || k_y<0 || k_y>=N[1] || k_z<0 || k_z>=N[2])
    return 0.0;

  return get(k_x,k_y,k_z);
}
double Volume::get_interpolated(double x,double y,double z) const
{return get(x,y,z);}

int Volume::export_ppm(const std::string& filename,const std::vector <double>& current_slice,int N1,int N2)
{
  int ok=0;
  FILE *fid_file;  
  
  //export in ppm
  fid_file = fopen(filename.data(),"w");
  if(fid_file==NULL)
    {cout<<"ERROR CANNOT CREATE "<<filename<<"\n";return (-1);}

  fprintf(fid_file,"P3\n");//magic number
  fprintf(fid_file,"# %s.ppm\n",filename.data());//name of the file in comment
  fprintf(fid_file,"%d %d\n",N1,N2);//size of the file
  fprintf(fid_file,"255\n");//max value

  int k1=0,k2=0;
  int value=0;
  for(k1=0;k1<N1;k1++)
    for(k2=0;k2<N2;k2++)
      {
	value = int(fmin(fmax(current_slice[k1+k2*N2],0),255));
	fprintf(fid_file,"%d %d %d \n",value,value,value);
    }

  ok+=fclose(fid_file);
  return ok;

}
int Volume::export_ppm(const char* filename,const std::vector <double>& current_slice,int N1,int N2)
{return export_ppm(std::string(filename),current_slice,N1,N2);}


int Volume::export_coff(const std::string& filename,const std::vector <double>& vertex,const std::vector <int>& connectivity,const std::vector <double>& color)
{
  FILE *fid;
  fid = fopen(filename.data(),"w");
  if(fid==NULL)
    {cout<<"ERROR CANNOT CREATE "<<filename<<"\n";return (-1);}

  int N_vertex=vertex.size()/3;
  int N_triangle=connectivity.size()/3;

  if(color.size()!=vertex.size()/3)
    {printf("Error in export coff, color.size()=%d and vertex.size()=%d\n",color.size(),vertex.size());exit(-1);}

  fprintf(fid,"COFF\n");
  fprintf(fid,"%d %d %d\n",N_vertex,N_triangle,0);
  for(int k_vertex=0;k_vertex<N_vertex;k_vertex++)
    fprintf(fid,"%f %f %f   %f %f %f 1.0\n",vertex[3*k_vertex+0],vertex[3*k_vertex+1],vertex[3*k_vertex+2],
 	    color[k_vertex]/255.0,color[k_vertex]/255.0,color[k_vertex]/255.0);
  for(int k_connnectivity=0;k_connnectivity<N_triangle;k_connnectivity++)
    fprintf(fid,"3 %d %d %d\n",connectivity[3*k_connnectivity+0],connectivity[3*k_connnectivity+1],connectivity[3*k_connnectivity+2]);

  if(fclose(fid)!=0)
    {printf("Error in export coff file %s\n",filename.data());exit(-1);}

  return 0;
}

int Volume::export_coff(const char* filename,const std::vector <double>& vertex,const std::vector <int>& connectivity,const std::vector <double>& color)
{return export_coff(std::string(filename),vertex,connectivity,color);}

int Volume::export_off(const std::string& filename,const std::vector <double>& vertex,const std::vector <int>& connectivity)
{
  FILE *fid;
  fid = fopen(filename.data(),"w");
  if(fid==NULL)
    {cout<<"ERROR CANNOT CREATE "<<filename<<"\n";return (-1);}

  int N_vertex=vertex.size()/3;
  int N_triangle=connectivity.size()/3;

  fprintf(fid,"OFF\n");
  fprintf(fid,"%d %d %d\n",N_vertex,N_triangle,0);
  for(int k_vertex=0;k_vertex<N_vertex;k_vertex++)
    fprintf(fid,"%f %f %f \n",vertex[3*k_vertex+0],vertex[3*k_vertex+1],vertex[3*k_vertex+2]);
  for(int k_connnectivity=0;k_connnectivity<N_triangle;k_connnectivity++)
    fprintf(fid,"3 %d %d %d\n",connectivity[3*k_connnectivity+0],connectivity[3*k_connnectivity+1],connectivity[3*k_connnectivity+2]);

  if(fclose(fid)!=0)
    {printf("Error in export off file %s\n",filename.data());exit(-1);}

  return 0;
}

int Volume::export_off(const char* filename,const std::vector <double>& vertex,const std::vector <int>& connectivity)
{return export_off(std::string(filename),vertex,connectivity);}


int Volume::resample(double d_L)
{

  return 0;
}


int Volume::isosurface(double value,std::vector <double>* vertex,std::vector <int>* connectivity) const
{
  // one cell
  std::vector <double> temp_cell(8);
  int type_of_cube=0;

  double k_1=0,k_2=0,k_3=0;
  for(k_1=0;k_1<N[0];k_1++)
    for(k_2=0;k_2<N[1];k_2++)
      for(k_3=0;k_3<N[2];k_3++)
	{
	  
 	  temp_cell[0] = get(k_1+0,k_2+0,k_3+0);
 	  temp_cell[1] = get(k_1+1,k_2+0,k_3+0);
 	  temp_cell[2] = get(k_1+1,k_2+0,k_3+1);
 	  temp_cell[3] = get(k_1+0,k_2+0,k_3+1);

 	  temp_cell[4] = get(k_1+0,k_2+1,k_3+0);
 	  temp_cell[5] = get(k_1+1,k_2+1,k_3+0);
 	  temp_cell[6] = get(k_1+1,k_2+1,k_3+1);
 	  temp_cell[7] = get(k_1+0,k_2+1,k_3+1);

	  type_of_cube=0;
	  if(temp_cell[0] <= value) type_of_cube |= 0x01;
	  if(temp_cell[1] <= value) type_of_cube |= 0x02;
	  if(temp_cell[2] <= value) type_of_cube |= 0x04;
	  if(temp_cell[3] <= value) type_of_cube |= 0x08;
	  if(temp_cell[4] <= value) type_of_cube |= 0x10;
	  if(temp_cell[5] <= value) type_of_cube |= 0x20;
	  if(temp_cell[6] <= value) type_of_cube |= 0x40;
	  if(temp_cell[7] <= value) type_of_cube |= 0x80;

	  if(EdgeTable[type_of_cube]!=0) // we have a polygon
	    create_polygon(temp_cell,value,type_of_cube,V_3D(k_1,k_2,k_3),vertex,connectivity);
	}

  

  return 0;
}

int Volume::create_polygon(const std::vector <double>& cell_values,double value,int type_of_cube,V_3D position,std::vector <double>* vertex,std::vector <int>* connectivity) const
{

  std::vector <V_3D> edge_position(12);
  if(EdgeTable[type_of_cube] & 0x001) //edge 0->1
    edge_position[ 0] = position+V_3D(0.0,0.0,0.0);
  if(EdgeTable[type_of_cube] & 0x002) //edge 1->2
    edge_position[ 1] = position+V_3D(1.0,0.0,0.0);
  if(EdgeTable[type_of_cube] & 0x004) //edge 2->3
    edge_position[ 2] = position+V_3D(1.0,0.0,1.0);
  if(EdgeTable[type_of_cube] & 0x008) //edge 3->0
    edge_position[ 3] = position+V_3D(0.0,0.0,1.0);
  
  if(EdgeTable[type_of_cube] & 0x010) //edge 4->5
    edge_position[ 4] = position+V_3D(0.0,1.0,0.0);
  if(EdgeTable[type_of_cube] & 0x020) //edge 5->6
    edge_position[ 5] = position+V_3D(1.0,1.0,0.0);
  if(EdgeTable[type_of_cube] & 0x040) //edge 6->7
    edge_position[ 6] = position+V_3D(1.0,1.0,1.0);
  if(EdgeTable[type_of_cube] & 0x080) //edge 7->4
    edge_position[ 7] = position+V_3D(0.0,1.0,1.0);
  
  if(EdgeTable[type_of_cube] & 0x100) //edge 0->4
    edge_position[ 8] = position+V_3D(0.0,0.0,0.0);
  if(EdgeTable[type_of_cube] & 0x200) //edge 1->5
    edge_position[ 9] = position+V_3D(1.0,0.0,0.0);
  if(EdgeTable[type_of_cube] & 0x400) //edge 2->6
    edge_position[10] = position+V_3D(1.0,0.0,1.0);
  if(EdgeTable[type_of_cube] & 0x800) //edge 3->7
    edge_position[11] = position+V_3D(0.0,0.0,1.0);
  

  
  // triangulation
  int triangulation_index=0;
  int k_v=0;
  for(triangulation_index=0;TriangulationTable[type_of_cube][triangulation_index]!=-1;triangulation_index+=3)
    for(k_v=0;k_v<3;k_v++)
      add_vertex_in_mesh(vertex,connectivity,edge_position[TriangulationTable[type_of_cube][triangulation_index+k_v]]);

  return 0;
}




int Volume::add_vertex_in_mesh(std::vector <double>* vertex,std::vector <int>* connectivity,const V_3D& x) const
{
  for(unsigned int k_dim=0;k_dim<3;k_dim++)
    vertex->push_back(x[k_dim]); 
  connectivity->push_back(vertex->size()/3-1);
  return vertex->size()/3-1;
}

// Helper indexation table for the marching cube
int Volume::EdgeTable[256]={
	0x0  , 0x109, 0x203, 0x30a, 0x406, 0x50f, 0x605, 0x70c,
	0x80c, 0x905, 0xa0f, 0xb06, 0xc0a, 0xd03, 0xe09, 0xf00,
	0x190, 0x99 , 0x393, 0x29a, 0x596, 0x49f, 0x795, 0x69c,
	0x99c, 0x895, 0xb9f, 0xa96, 0xd9a, 0xc93, 0xf99, 0xe90,
	0x230, 0x339, 0x33 , 0x13a, 0x636, 0x73f, 0x435, 0x53c,
	0xa3c, 0xb35, 0x83f, 0x936, 0xe3a, 0xf33, 0xc39, 0xd30,
	0x3a0, 0x2a9, 0x1a3, 0xaa , 0x7a6, 0x6af, 0x5a5, 0x4ac,
	0xbac, 0xaa5, 0x9af, 0x8a6, 0xfaa, 0xea3, 0xda9, 0xca0,
	0x460, 0x569, 0x663, 0x76a, 0x66 , 0x16f, 0x265, 0x36c,
	0xc6c, 0xd65, 0xe6f, 0xf66, 0x86a, 0x963, 0xa69, 0xb60,
	0x5f0, 0x4f9, 0x7f3, 0x6fa, 0x1f6, 0xff , 0x3f5, 0x2fc,
	0xdfc, 0xcf5, 0xfff, 0xef6, 0x9fa, 0x8f3, 0xbf9, 0xaf0,
	0x650, 0x759, 0x453, 0x55a, 0x256, 0x35f, 0x55 , 0x15c,
	0xe5c, 0xf55, 0xc5f, 0xd56, 0xa5a, 0xb53, 0x859, 0x950,
	0x7c0, 0x6c9, 0x5c3, 0x4ca, 0x3c6, 0x2cf, 0x1c5, 0xcc ,
	0xfcc, 0xec5, 0xdcf, 0xcc6, 0xbca, 0xac3, 0x9c9, 0x8c0,
	0x8c0, 0x9c9, 0xac3, 0xbca, 0xcc6, 0xdcf, 0xec5, 0xfcc,
	0xcc , 0x1c5, 0x2cf, 0x3c6, 0x4ca, 0x5c3, 0x6c9, 0x7c0,
	0x950, 0x859, 0xb53, 0xa5a, 0xd56, 0xc5f, 0xf55, 0xe5c,
	0x15c, 0x55 , 0x35f, 0x256, 0x55a, 0x453, 0x759, 0x650,
	0xaf0, 0xbf9, 0x8f3, 0x9fa, 0xef6, 0xfff, 0xcf5, 0xdfc,
	0x2fc, 0x3f5, 0xff , 0x1f6, 0x6fa, 0x7f3, 0x4f9, 0x5f0,
	0xb60, 0xa69, 0x963, 0x86a, 0xf66, 0xe6f, 0xd65, 0xc6c,
	0x36c, 0x265, 0x16f, 0x66 , 0x76a, 0x663, 0x569, 0x460,
	0xca0, 0xda9, 0xea3, 0xfaa, 0x8a6, 0x9af, 0xaa5, 0xbac,
	0x4ac, 0x5a5, 0x6af, 0x7a6, 0xaa , 0x1a3, 0x2a9, 0x3a0,
	0xd30, 0xc39, 0xf33, 0xe3a, 0x936, 0x83f, 0xb35, 0xa3c,
	0x53c, 0x435, 0x73f, 0x636, 0x13a, 0x33 , 0x339, 0x230,
	0xe90, 0xf99, 0xc93, 0xd9a, 0xa96, 0xb9f, 0x895, 0x99c,
	0x69c, 0x795, 0x49f, 0x596, 0x29a, 0x393, 0x99 , 0x190,
	0xf00, 0xe09, 0xd03, 0xc0a, 0xb06, 0xa0f, 0x905, 0x80c,
	0x70c, 0x605, 0x50f, 0x406, 0x30a, 0x203, 0x109, 0x0};

int Volume::TriangulationTable[256][16] =
	{{-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 1, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 8, 3, 9, 8, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 3, 1, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{9, 2, 10, 0, 2, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{2, 8, 3, 2, 10, 8, 10, 9, 8, -1, -1, -1, -1, -1, -1, -1},
	{3, 11, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 11, 2, 8, 11, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 9, 0, 2, 3, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 11, 2, 1, 9, 11, 9, 8, 11, -1, -1, -1, -1, -1, -1, -1},
	{3, 10, 1, 11, 10, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 10, 1, 0, 8, 10, 8, 11, 10, -1, -1, -1, -1, -1, -1, -1},
	{3, 9, 0, 3, 11, 9, 11, 10, 9, -1, -1, -1, -1, -1, -1, -1},
	{9, 8, 10, 10, 8, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 7, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 3, 0, 7, 3, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 1, 9, 8, 4, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 1, 9, 4, 7, 1, 7, 3, 1, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 10, 8, 4, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{3, 4, 7, 3, 0, 4, 1, 2, 10, -1, -1, -1, -1, -1, -1, -1},
	{9, 2, 10, 9, 0, 2, 8, 4, 7, -1, -1, -1, -1, -1, -1, -1},
	{2, 10, 9, 2, 9, 7, 2, 7, 3, 7, 9, 4, -1, -1, -1, -1},
	{8, 4, 7, 3, 11, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{11, 4, 7, 11, 2, 4, 2, 0, 4, -1, -1, -1, -1, -1, -1, -1},
	{9, 0, 1, 8, 4, 7, 2, 3, 11, -1, -1, -1, -1, -1, -1, -1},
	{4, 7, 11, 9, 4, 11, 9, 11, 2, 9, 2, 1, -1, -1, -1, -1},
	{3, 10, 1, 3, 11, 10, 7, 8, 4, -1, -1, -1, -1, -1, -1, -1},
	{1, 11, 10, 1, 4, 11, 1, 0, 4, 7, 11, 4, -1, -1, -1, -1},
	{4, 7, 8, 9, 0, 11, 9, 11, 10, 11, 0, 3, -1, -1, -1, -1},
	{4, 7, 11, 4, 11, 9, 9, 11, 10, -1, -1, -1, -1, -1, -1, -1},
	{9, 5, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{9, 5, 4, 0, 8, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 5, 4, 1, 5, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{8, 5, 4, 8, 3, 5, 3, 1, 5, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 10, 9, 5, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{3, 0, 8, 1, 2, 10, 4, 9, 5, -1, -1, -1, -1, -1, -1, -1},
	{5, 2, 10, 5, 4, 2, 4, 0, 2, -1, -1, -1, -1, -1, -1, -1},
	{2, 10, 5, 3, 2, 5, 3, 5, 4, 3, 4, 8, -1, -1, -1, -1},
	{9, 5, 4, 2, 3, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 11, 2, 0, 8, 11, 4, 9, 5, -1, -1, -1, -1, -1, -1, -1},
	{0, 5, 4, 0, 1, 5, 2, 3, 11, -1, -1, -1, -1, -1, -1, -1},
	{2, 1, 5, 2, 5, 8, 2, 8, 11, 4, 8, 5, -1, -1, -1, -1},
	{10, 3, 11, 10, 1, 3, 9, 5, 4, -1, -1, -1, -1, -1, -1, -1},
	{4, 9, 5, 0, 8, 1, 8, 10, 1, 8, 11, 10, -1, -1, -1, -1},
	{5, 4, 0, 5, 0, 11, 5, 11, 10, 11, 0, 3, -1, -1, -1, -1},
	{5, 4, 8, 5, 8, 10, 10, 8, 11, -1, -1, -1, -1, -1, -1, -1},
	{9, 7, 8, 5, 7, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{9, 3, 0, 9, 5, 3, 5, 7, 3, -1, -1, -1, -1, -1, -1, -1},
	{0, 7, 8, 0, 1, 7, 1, 5, 7, -1, -1, -1, -1, -1, -1, -1},
	{1, 5, 3, 3, 5, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{9, 7, 8, 9, 5, 7, 10, 1, 2, -1, -1, -1, -1, -1, -1, -1},
	{10, 1, 2, 9, 5, 0, 5, 3, 0, 5, 7, 3, -1, -1, -1, -1},
	{8, 0, 2, 8, 2, 5, 8, 5, 7, 10, 5, 2, -1, -1, -1, -1},
	{2, 10, 5, 2, 5, 3, 3, 5, 7, -1, -1, -1, -1, -1, -1, -1},
	{7, 9, 5, 7, 8, 9, 3, 11, 2, -1, -1, -1, -1, -1, -1, -1},
	{9, 5, 7, 9, 7, 2, 9, 2, 0, 2, 7, 11, -1, -1, -1, -1},
	{2, 3, 11, 0, 1, 8, 1, 7, 8, 1, 5, 7, -1, -1, -1, -1},
	{11, 2, 1, 11, 1, 7, 7, 1, 5, -1, -1, -1, -1, -1, -1, -1},
	{9, 5, 8, 8, 5, 7, 10, 1, 3, 10, 3, 11, -1, -1, -1, -1},
	{5, 7, 0, 5, 0, 9, 7, 11, 0, 1, 0, 10, 11, 10, 0, -1},
	{11, 10, 0, 11, 0, 3, 10, 5, 0, 8, 0, 7, 5, 7, 0, -1},
	{11, 10, 5, 7, 11, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{10, 6, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 3, 5, 10, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{9, 0, 1, 5, 10, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 8, 3, 1, 9, 8, 5, 10, 6, -1, -1, -1, -1, -1, -1, -1},
	{1, 6, 5, 2, 6, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 6, 5, 1, 2, 6, 3, 0, 8, -1, -1, -1, -1, -1, -1, -1},
	{9, 6, 5, 9, 0, 6, 0, 2, 6, -1, -1, -1, -1, -1, -1, -1},
	{5, 9, 8, 5, 8, 2, 5, 2, 6, 3, 2, 8, -1, -1, -1, -1},
	{2, 3, 11, 10, 6, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{11, 0, 8, 11, 2, 0, 10, 6, 5, -1, -1, -1, -1, -1, -1, -1},
	{0, 1, 9, 2, 3, 11, 5, 10, 6, -1, -1, -1, -1, -1, -1, -1},
	{5, 10, 6, 1, 9, 2, 9, 11, 2, 9, 8, 11, -1, -1, -1, -1},
	{6, 3, 11, 6, 5, 3, 5, 1, 3, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 11, 0, 11, 5, 0, 5, 1, 5, 11, 6, -1, -1, -1, -1},
	{3, 11, 6, 0, 3, 6, 0, 6, 5, 0, 5, 9, -1, -1, -1, -1},
	{6, 5, 9, 6, 9, 11, 11, 9, 8, -1, -1, -1, -1, -1, -1, -1},
	{5, 10, 6, 4, 7, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 3, 0, 4, 7, 3, 6, 5, 10, -1, -1, -1, -1, -1, -1, -1},
	{1, 9, 0, 5, 10, 6, 8, 4, 7, -1, -1, -1, -1, -1, -1, -1},
	{10, 6, 5, 1, 9, 7, 1, 7, 3, 7, 9, 4, -1, -1, -1, -1},
	{6, 1, 2, 6, 5, 1, 4, 7, 8, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 5, 5, 2, 6, 3, 0, 4, 3, 4, 7, -1, -1, -1, -1},
	{8, 4, 7, 9, 0, 5, 0, 6, 5, 0, 2, 6, -1, -1, -1, -1},
	{7, 3, 9, 7, 9, 4, 3, 2, 9, 5, 9, 6, 2, 6, 9, -1},
	{3, 11, 2, 7, 8, 4, 10, 6, 5, -1, -1, -1, -1, -1, -1, -1},
	{5, 10, 6, 4, 7, 2, 4, 2, 0, 2, 7, 11, -1, -1, -1, -1},
	{0, 1, 9, 4, 7, 8, 2, 3, 11, 5, 10, 6, -1, -1, -1, -1},
	{9, 2, 1, 9, 11, 2, 9, 4, 11, 7, 11, 4, 5, 10, 6, -1},
	{8, 4, 7, 3, 11, 5, 3, 5, 1, 5, 11, 6, -1, -1, -1, -1},
	{5, 1, 11, 5, 11, 6, 1, 0, 11, 7, 11, 4, 0, 4, 11, -1},
	{0, 5, 9, 0, 6, 5, 0, 3, 6, 11, 6, 3, 8, 4, 7, -1},
	{6, 5, 9, 6, 9, 11, 4, 7, 9, 7, 11, 9, -1, -1, -1, -1},
	{10, 4, 9, 6, 4, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 10, 6, 4, 9, 10, 0, 8, 3, -1, -1, -1, -1, -1, -1, -1},
	{10, 0, 1, 10, 6, 0, 6, 4, 0, -1, -1, -1, -1, -1, -1, -1},
	{8, 3, 1, 8, 1, 6, 8, 6, 4, 6, 1, 10, -1, -1, -1, -1},
	{1, 4, 9, 1, 2, 4, 2, 6, 4, -1, -1, -1, -1, -1, -1, -1},
	{3, 0, 8, 1, 2, 9, 2, 4, 9, 2, 6, 4, -1, -1, -1, -1},
	{0, 2, 4, 4, 2, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{8, 3, 2, 8, 2, 4, 4, 2, 6, -1, -1, -1, -1, -1, -1, -1},
	{10, 4, 9, 10, 6, 4, 11, 2, 3, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 2, 2, 8, 11, 4, 9, 10, 4, 10, 6, -1, -1, -1, -1},
	{3, 11, 2, 0, 1, 6, 0, 6, 4, 6, 1, 10, -1, -1, -1, -1},
	{6, 4, 1, 6, 1, 10, 4, 8, 1, 2, 1, 11, 8, 11, 1, -1},
	{9, 6, 4, 9, 3, 6, 9, 1, 3, 11, 6, 3, -1, -1, -1, -1},
	{8, 11, 1, 8, 1, 0, 11, 6, 1, 9, 1, 4, 6, 4, 1, -1},
	{3, 11, 6, 3, 6, 0, 0, 6, 4, -1, -1, -1, -1, -1, -1, -1},
	{6, 4, 8, 11, 6, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{7, 10, 6, 7, 8, 10, 8, 9, 10, -1, -1, -1, -1, -1, -1, -1},
	{0, 7, 3, 0, 10, 7, 0, 9, 10, 6, 7, 10, -1, -1, -1, -1},
	{10, 6, 7, 1, 10, 7, 1, 7, 8, 1, 8, 0, -1, -1, -1, -1},
	{10, 6, 7, 10, 7, 1, 1, 7, 3, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 6, 1, 6, 8, 1, 8, 9, 8, 6, 7, -1, -1, -1, -1},
	{2, 6, 9, 2, 9, 1, 6, 7, 9, 0, 9, 3, 7, 3, 9, -1},
	{7, 8, 0, 7, 0, 6, 6, 0, 2, -1, -1, -1, -1, -1, -1, -1},
	{7, 3, 2, 6, 7, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{2, 3, 11, 10, 6, 8, 10, 8, 9, 8, 6, 7, -1, -1, -1, -1},
	{2, 0, 7, 2, 7, 11, 0, 9, 7, 6, 7, 10, 9, 10, 7, -1},
	{1, 8, 0, 1, 7, 8, 1, 10, 7, 6, 7, 10, 2, 3, 11, -1},
	{11, 2, 1, 11, 1, 7, 10, 6, 1, 6, 7, 1, -1, -1, -1, -1},
	{8, 9, 6, 8, 6, 7, 9, 1, 6, 11, 6, 3, 1, 3, 6, -1},
	{0, 9, 1, 11, 6, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{7, 8, 0, 7, 0, 6, 3, 11, 0, 11, 6, 0, -1, -1, -1, -1},
	{7, 11, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{7, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{3, 0, 8, 11, 7, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 1, 9, 11, 7, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{8, 1, 9, 8, 3, 1, 11, 7, 6, -1, -1, -1, -1, -1, -1, -1},
	{10, 1, 2, 6, 11, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 10, 3, 0, 8, 6, 11, 7, -1, -1, -1, -1, -1, -1, -1},
	{2, 9, 0, 2, 10, 9, 6, 11, 7, -1, -1, -1, -1, -1, -1, -1},
	{6, 11, 7, 2, 10, 3, 10, 8, 3, 10, 9, 8, -1, -1, -1, -1},
	{7, 2, 3, 6, 2, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{7, 0, 8, 7, 6, 0, 6, 2, 0, -1, -1, -1, -1, -1, -1, -1},
	{2, 7, 6, 2, 3, 7, 0, 1, 9, -1, -1, -1, -1, -1, -1, -1},
	{1, 6, 2, 1, 8, 6, 1, 9, 8, 8, 7, 6, -1, -1, -1, -1},
	{10, 7, 6, 10, 1, 7, 1, 3, 7, -1, -1, -1, -1, -1, -1, -1},
	{10, 7, 6, 1, 7, 10, 1, 8, 7, 1, 0, 8, -1, -1, -1, -1},
	{0, 3, 7, 0, 7, 10, 0, 10, 9, 6, 10, 7, -1, -1, -1, -1},
	{7, 6, 10, 7, 10, 8, 8, 10, 9, -1, -1, -1, -1, -1, -1, -1},
	{6, 8, 4, 11, 8, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{3, 6, 11, 3, 0, 6, 0, 4, 6, -1, -1, -1, -1, -1, -1, -1},
	{8, 6, 11, 8, 4, 6, 9, 0, 1, -1, -1, -1, -1, -1, -1, -1},
	{9, 4, 6, 9, 6, 3, 9, 3, 1, 11, 3, 6, -1, -1, -1, -1},
	{6, 8, 4, 6, 11, 8, 2, 10, 1, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 10, 3, 0, 11, 0, 6, 11, 0, 4, 6, -1, -1, -1, -1},
	{4, 11, 8, 4, 6, 11, 0, 2, 9, 2, 10, 9, -1, -1, -1, -1},
	{10, 9, 3, 10, 3, 2, 9, 4, 3, 11, 3, 6, 4, 6, 3, -1},
	{8, 2, 3, 8, 4, 2, 4, 6, 2, -1, -1, -1, -1, -1, -1, -1},
	{0, 4, 2, 4, 6, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 9, 0, 2, 3, 4, 2, 4, 6, 4, 3, 8, -1, -1, -1, -1},
	{1, 9, 4, 1, 4, 2, 2, 4, 6, -1, -1, -1, -1, -1, -1, -1},
	{8, 1, 3, 8, 6, 1, 8, 4, 6, 6, 10, 1, -1, -1, -1, -1},
	{10, 1, 0, 10, 0, 6, 6, 0, 4, -1, -1, -1, -1, -1, -1, -1},
	{4, 6, 3, 4, 3, 8, 6, 10, 3, 0, 3, 9, 10, 9, 3, -1},
	{10, 9, 4, 6, 10, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 9, 5, 7, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 3, 4, 9, 5, 11, 7, 6, -1, -1, -1, -1, -1, -1, -1},
	{5, 0, 1, 5, 4, 0, 7, 6, 11, -1, -1, -1, -1, -1, -1, -1},
	{11, 7, 6, 8, 3, 4, 3, 5, 4, 3, 1, 5, -1, -1, -1, -1},
	{9, 5, 4, 10, 1, 2, 7, 6, 11, -1, -1, -1, -1, -1, -1, -1},
	{6, 11, 7, 1, 2, 10, 0, 8, 3, 4, 9, 5, -1, -1, -1, -1},
	{7, 6, 11, 5, 4, 10, 4, 2, 10, 4, 0, 2, -1, -1, -1, -1},
	{3, 4, 8, 3, 5, 4, 3, 2, 5, 10, 5, 2, 11, 7, 6, -1},
	{7, 2, 3, 7, 6, 2, 5, 4, 9, -1, -1, -1, -1, -1, -1, -1},
	{9, 5, 4, 0, 8, 6, 0, 6, 2, 6, 8, 7, -1, -1, -1, -1},
	{3, 6, 2, 3, 7, 6, 1, 5, 0, 5, 4, 0, -1, -1, -1, -1},
	{6, 2, 8, 6, 8, 7, 2, 1, 8, 4, 8, 5, 1, 5, 8, -1},
	{9, 5, 4, 10, 1, 6, 1, 7, 6, 1, 3, 7, -1, -1, -1, -1},
	{1, 6, 10, 1, 7, 6, 1, 0, 7, 8, 7, 0, 9, 5, 4, -1},
	{4, 0, 10, 4, 10, 5, 0, 3, 10, 6, 10, 7, 3, 7, 10, -1},
	{7, 6, 10, 7, 10, 8, 5, 4, 10, 4, 8, 10, -1, -1, -1, -1},
	{6, 9, 5, 6, 11, 9, 11, 8, 9, -1, -1, -1, -1, -1, -1, -1},
	{3, 6, 11, 0, 6, 3, 0, 5, 6, 0, 9, 5, -1, -1, -1, -1},
	{0, 11, 8, 0, 5, 11, 0, 1, 5, 5, 6, 11, -1, -1, -1, -1},
	{6, 11, 3, 6, 3, 5, 5, 3, 1, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 10, 9, 5, 11, 9, 11, 8, 11, 5, 6, -1, -1, -1, -1},
	{0, 11, 3, 0, 6, 11, 0, 9, 6, 5, 6, 9, 1, 2, 10, -1},
	{11, 8, 5, 11, 5, 6, 8, 0, 5, 10, 5, 2, 0, 2, 5, -1},
	{6, 11, 3, 6, 3, 5, 2, 10, 3, 10, 5, 3, -1, -1, -1, -1},
	{5, 8, 9, 5, 2, 8, 5, 6, 2, 3, 8, 2, -1, -1, -1, -1},
	{9, 5, 6, 9, 6, 0, 0, 6, 2, -1, -1, -1, -1, -1, -1, -1},
	{1, 5, 8, 1, 8, 0, 5, 6, 8, 3, 8, 2, 6, 2, 8, -1},
	{1, 5, 6, 2, 1, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 3, 6, 1, 6, 10, 3, 8, 6, 5, 6, 9, 8, 9, 6, -1},
	{10, 1, 0, 10, 0, 6, 9, 5, 0, 5, 6, 0, -1, -1, -1, -1},
	{0, 3, 8, 5, 6, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{10, 5, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{11, 5, 10, 7, 5, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{11, 5, 10, 11, 7, 5, 8, 3, 0, -1, -1, -1, -1, -1, -1, -1},
	{5, 11, 7, 5, 10, 11, 1, 9, 0, -1, -1, -1, -1, -1, -1, -1},
	{10, 7, 5, 10, 11, 7, 9, 8, 1, 8, 3, 1, -1, -1, -1, -1},
	{11, 1, 2, 11, 7, 1, 7, 5, 1, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 3, 1, 2, 7, 1, 7, 5, 7, 2, 11, -1, -1, -1, -1},
	{9, 7, 5, 9, 2, 7, 9, 0, 2, 2, 11, 7, -1, -1, -1, -1},
	{7, 5, 2, 7, 2, 11, 5, 9, 2, 3, 2, 8, 9, 8, 2, -1},
	{2, 5, 10, 2, 3, 5, 3, 7, 5, -1, -1, -1, -1, -1, -1, -1},
	{8, 2, 0, 8, 5, 2, 8, 7, 5, 10, 2, 5, -1, -1, -1, -1},
	{9, 0, 1, 5, 10, 3, 5, 3, 7, 3, 10, 2, -1, -1, -1, -1},
	{9, 8, 2, 9, 2, 1, 8, 7, 2, 10, 2, 5, 7, 5, 2, -1},
	{1, 3, 5, 3, 7, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 7, 0, 7, 1, 1, 7, 5, -1, -1, -1, -1, -1, -1, -1},
	{9, 0, 3, 9, 3, 5, 5, 3, 7, -1, -1, -1, -1, -1, -1, -1},
	{9, 8, 7, 5, 9, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{5, 8, 4, 5, 10, 8, 10, 11, 8, -1, -1, -1, -1, -1, -1, -1},
	{5, 0, 4, 5, 11, 0, 5, 10, 11, 11, 3, 0, -1, -1, -1, -1},
	{0, 1, 9, 8, 4, 10, 8, 10, 11, 10, 4, 5, -1, -1, -1, -1},
	{10, 11, 4, 10, 4, 5, 11, 3, 4, 9, 4, 1, 3, 1, 4, -1},
	{2, 5, 1, 2, 8, 5, 2, 11, 8, 4, 5, 8, -1, -1, -1, -1},
	{0, 4, 11, 0, 11, 3, 4, 5, 11, 2, 11, 1, 5, 1, 11, -1},
	{0, 2, 5, 0, 5, 9, 2, 11, 5, 4, 5, 8, 11, 8, 5, -1},
	{9, 4, 5, 2, 11, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{2, 5, 10, 3, 5, 2, 3, 4, 5, 3, 8, 4, -1, -1, -1, -1},
	{5, 10, 2, 5, 2, 4, 4, 2, 0, -1, -1, -1, -1, -1, -1, -1},
	{3, 10, 2, 3, 5, 10, 3, 8, 5, 4, 5, 8, 0, 1, 9, -1},
	{5, 10, 2, 5, 2, 4, 1, 9, 2, 9, 4, 2, -1, -1, -1, -1},
	{8, 4, 5, 8, 5, 3, 3, 5, 1, -1, -1, -1, -1, -1, -1, -1},
	{0, 4, 5, 1, 0, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{8, 4, 5, 8, 5, 3, 9, 0, 5, 0, 3, 5, -1, -1, -1, -1},
	{9, 4, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 11, 7, 4, 9, 11, 9, 10, 11, -1, -1, -1, -1, -1, -1, -1},
	{0, 8, 3, 4, 9, 7, 9, 11, 7, 9, 10, 11, -1, -1, -1, -1},
	{1, 10, 11, 1, 11, 4, 1, 4, 0, 7, 4, 11, -1, -1, -1, -1},
	{3, 1, 4, 3, 4, 8, 1, 10, 4, 7, 4, 11, 10, 11, 4, -1},
	{4, 11, 7, 9, 11, 4, 9, 2, 11, 9, 1, 2, -1, -1, -1, -1},
	{9, 7, 4, 9, 11, 7, 9, 1, 11, 2, 11, 1, 0, 8, 3, -1},
	{11, 7, 4, 11, 4, 2, 2, 4, 0, -1, -1, -1, -1, -1, -1, -1},
	{11, 7, 4, 11, 4, 2, 8, 3, 4, 3, 2, 4, -1, -1, -1, -1},
	{2, 9, 10, 2, 7, 9, 2, 3, 7, 7, 4, 9, -1, -1, -1, -1},
	{9, 10, 7, 9, 7, 4, 10, 2, 7, 8, 7, 0, 2, 0, 7, -1},
	{3, 7, 10, 3, 10, 2, 7, 4, 10, 1, 10, 0, 4, 0, 10, -1},
	{1, 10, 2, 8, 7, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 9, 1, 4, 1, 7, 7, 1, 3, -1, -1, -1, -1, -1, -1, -1},
	{4, 9, 1, 4, 1, 7, 0, 8, 1, 8, 7, 1, -1, -1, -1, -1},
	{4, 0, 3, 7, 4, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{4, 8, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{9, 10, 8, 10, 11, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{3, 0, 9, 3, 9, 11, 11, 9, 10, -1, -1, -1, -1, -1, -1, -1},
	{0, 1, 10, 0, 10, 8, 8, 10, 11, -1, -1, -1, -1, -1, -1, -1},
	{3, 1, 10, 11, 3, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 2, 11, 1, 11, 9, 9, 11, 8, -1, -1, -1, -1, -1, -1, -1},
	{3, 0, 9, 3, 9, 11, 1, 2, 9, 2, 11, 9, -1, -1, -1, -1},
	{0, 2, 11, 8, 0, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{3, 2, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{2, 3, 8, 2, 8, 10, 10, 8, 9, -1, -1, -1, -1, -1, -1, -1},
	{9, 10, 2, 0, 9, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{2, 3, 8, 2, 8, 10, 0, 1, 8, 1, 10, 8, -1, -1, -1, -1},
	{1, 10, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{1, 3, 8, 9, 1, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 9, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{0, 3, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
	{-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}};
